@@ -86,7 +86,7 @@ class AndroidLibrary(object):
         self._password = password
 
     def start_emulator(self, avd_name, no_window=False,
-                       language="en", country="us", save_snapshot=False, retries=3):
+                       language="en", country="us", save_snapshot=False, retries=3, http_proxy=""):
         '''
         Starts the Android Emulator.
 
@@ -103,6 +103,10 @@ class AndroidLibrary(object):
         if not save_snapshot:
             args.append('-no-snapshot-save')
 
+        if len(http_proxy)>0:
+            args.append('-http-proxy')
+            args.append(http_proxy)
+
         logging.debug("$> %s", ' '.join(args))
 
         self._emulator_proc = subprocess.Popen(args)
@@ -220,9 +224,8 @@ class AndroidLibrary(object):
         '''
         Presses the back button.
         '''
-        result = self._perform_action("go_back")
-        assert result["success"] is True, "Could not press back button: %s" % (
-            result.get('message', 'No specific error message given'))
+        response = self._perform_action("go_back")
+        assert response["success"] is True, "Could not press back button:: %s" % (response["message"])
 
     def press_menu_button(self):
         '''
@@ -334,9 +337,12 @@ class AndroidLibrary(object):
             "main_activity",
             main_activity,
             "-e",
+            "test_server_port",
+            str(7102),
+            "-e",
             "class",
             "sh.calaba.instrumentationbackend.InstrumentationBackend",
-            "sh.calaba.android.test/sh.calaba.instrumentationbackend.CalabashInstrumentationTestRunner",
+            "%s.test/sh.calaba.instrumentationbackend.CalabashInstrumentationTestRunner" % package_name,
         ]
         self._testserver_proc = subprocess.Popen(args)
 
@@ -396,7 +402,12 @@ class AndroidLibrary(object):
         logging.error("<< %r", url)
         logging.error("<< %r", response.text)
         assert response.status_code == 200, "InstrumentationBackend sent status %d, expected 200" % response.status_code
-        return response.json
+        try:
+            response_decoded = json.loads(response.text)
+            return response_decoded
+        except ValueError:
+            return response.text
+        return "error"
 
     # BEGIN: STOLEN FROM SELENIUM2LIBRARY
 
@@ -462,9 +473,8 @@ class AndroidLibrary(object):
 
         `text` String that should be on the current screen
         '''
-        result = self._perform_action("assert_text", text, True)
-        assert result["success"] is True, "Screen does not contain text '%s': %s" % (
-            text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("assert_text", text, True)
+        assert response["success"] is True, "Screen does not contain text '%s': %s" % (text, response["message"])
 
     def screen_should_not_contain(self, text):
         '''
@@ -472,9 +482,8 @@ class AndroidLibrary(object):
 
         `text` String that should not be on the current screen
         '''
-        result = self._perform_action("assert_text", text, False)
-        assert result["success"] is True, "Screen does contain text '%s', but shouldn't have: %s" % (
-            text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("assert_text", text, False)
+        assert response["success"] is True, "Screen does contain text '%s', but shouldn't have: %s" % (text, response["message"])
 
     def touch_button(self, text):
         '''
@@ -482,9 +491,8 @@ class AndroidLibrary(object):
 
         `text` is the text the button that will be clicked contains
         '''
-        result = self._perform_action("press_button_with_text", text)
-        assert result["success"] is True, "Touching button failed '%s': %s" % (
-            text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("press_button_with_text", text)
+        assert response["success"] is True, "Touching button '%s' failed: %s" % (text, response["message"])
 
     def touch_text(self, text):
         '''
@@ -492,17 +500,15 @@ class AndroidLibrary(object):
 
         `text` is the text the button that will be clicked contains
         '''
-        result = self._perform_action("click_on_text", text)
-        assert result["success"] is True, "Touching text '%s' failed: %s" % (
-            text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("click_on_text", text)
+        assert response["success"] is True, "Touching text '%s' failed: %s" % (text, response["message"])
 
     def scroll_up(self):
         '''
         Scroll up
         '''
-        result = self._perform_action("scroll_up")
-        assert result["success"] is True, "Scrolling up failed '%s': %s" % (
-            result.text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("scroll_up")
+        assert response["success"] is True, "Scrolling up failed: %s" % (response["message"])
 
     def touch_position(self, percent_left, percent_top):
         '''
@@ -513,17 +519,15 @@ class AndroidLibrary(object):
         '''
         percent_left = int(percent_left)
         percent_top = int(percent_top)
-        result = self._perform_action("click_on_screen", percent_left, percent_top)
-        assert result["success"] is True, "Touching position failed '%s': %s" % (
-            result.text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("click_on_screen", percent_left, percent_top)
+        assert response["success"] is True, "Touching position %s, %s failed: %s" % (percent_left, percent_top, response["message"])
 
     def scroll_down(self):
         '''
         Scroll down
         '''
-        result = self._perform_action("scroll_down")
-        assert result["success"] is True, "Scrolling down failed '%s': %s" % (
-            result.text, result.get('message', 'No specific error message given'))
+        response = self._perform_action("scroll_down")
+        assert response["success"] is True, "Scrolling down failed: %s" % (response["message"])
 
     def _split_locator(self, locator, default_strategy="css"):
         try:
@@ -542,9 +546,8 @@ class AndroidLibrary(object):
         `value` the new value
         '''
         strategy, query = self._split_locator(locator)
-        result = self._perform_action("set_text", strategy, query, value)
-
-        assert result["success"] is True, "Setting webview text failed '%r'" % result
+        response = self._perform_action("set_text", strategy, query, value)
+        assert response["success"] is True, "Setting webview text failed: %s" % (response["message"])
 
     def touch_webview_element(self, locator):
         '''
@@ -553,8 +556,8 @@ class AndroidLibrary(object):
         `locator` locator for element to trigger a click event (only css locators are supported at the moment)
         '''
         strategy, query = self._split_locator(locator)
-        result = self._perform_action("touch", strategy, query)
-        assert result["success"] is True, "Touching Webview element failed: '%r'" % result
+        response = self._perform_action("touch", strategy, query)
+        assert response["success"] is True, "Touching Webview element '%s' failed: %s" % (locator, response["message"])
 
     def webview_scroll_to(self, locator):
         '''
@@ -562,8 +565,8 @@ class AndroidLibrary(object):
         `locator` locator for element to scroll to (only css locators are supported at the moment)
         '''
         strategy, query = self._split_locator(locator)
-        result = self._perform_action("scroll_to", strategy, query)
-        assert result["success"] is True, "Scrolling to Webview element failed: '%r'" % result
+        response = self._perform_action("scroll_to", strategy, query)
+        assert response["success"] is True, "Scrolling to Webview element '%s' failed: %s" % (locator, response["message"])
 
     def set_text(self, locator, value):
         '''
@@ -593,9 +596,8 @@ class AndroidLibrary(object):
             '", "'.join(api_names.keys()), strategy
         )
 
-        result = self._perform_action(api_names[strategy], value, query)
-
-        assert result["success"] is True, "Setting the text failed: %s" % result
+        response = self._perform_action(api_names[strategy], value, query)
+        assert response["success"] is True, "Setting the text '%s' failed: %s" % (locator, response["message"])
 
     def webview_should_contain(self, text):
         '''
@@ -603,24 +605,22 @@ class AndroidLibrary(object):
 
         `text` the text the webview should contain
         '''
-        r = self._perform_action("query", "css", "html")
-        c = json.loads(r["message"])
-        assert text in c[0]["textContent"], "Webview does not contain: %s" % text
+        response = self._perform_action("query", "css", "html")
+        assert text in response[0]["textContent"], "Webview does not contain '%s': %s" % (text, response["message"])
 
     def swipe_left(self):
         '''
         Performs a swipe gesture to the left
         '''
-        result = self._perform_action('swipe', 'left')
-
-        assert result["success"] is True, "Swiping left failed: %s" % result
+        response = self._perform_action('swipe', 'left')
+        assert response["success"] is True, "Swiping left failed: %s" % response["message"]
 
     def swipe_right(self):
         '''
         Performs a swipe gesture to the right
         '''
-        result = self._perform_action('swipe', 'right')
-        assert result["success"] is True, "Swiping right failed: %s" % result
+        response = self._perform_action('swipe', 'right')
+        assert response["success"] is True, "Swiping right failed: %s" % response["message"]
 
     def touch_view(self, locator):
         '''
@@ -629,8 +629,8 @@ class AndroidLibrary(object):
         `locator` which view will be touched. Valid locators are '<string>' or'desc=<string>' for an imageButton with a contentDescription set.
         '''
         strategy, query = self._split_locator(locator, "desc")
-        result = self._perform_action('click_on_view_by_description', query)
-        assert result["success"] is True, "Click on view failed: %s" % result
+        response = self._perform_action('click_on_view_by_description', query)
+        assert response["success"] is True, "Click on view failed: %s" % response["message"]
 
     def touch_image_button(self, locator):
         '''
@@ -651,5 +651,5 @@ class AndroidLibrary(object):
         elif strategy == "desc":
             action = "press_image_button_description"
 
-        result = self._perform_action(action, query)
-        assert result["success"] is True, "Touching image button failed: %s" % result
+        response = self._perform_action(action, query)
+        assert response["success"] is True, "Touching image '%s' failed: %s" % (locator,response["message"])
